

function pop_ERSG( EEG, typeproc )


if EEG.trials == 1 
    error('You must extract epochs first');
    return;
end

if typeproc 
    if isempty(EEG.icaact)
        error('You must run ICA first');
        return;
    end
end

nyquist = EEG.srate / 2;
geometry = { [1 0.5 0.5] [1 0.5 0.5] [1 0.5 0.5] [1 0.5 0.5] [1 0.5 0.5] [1 0.5 0.5] [1 0.5 0.5] [1 0.5 0.5] [1 0.5 0.5] [1 0.5 0.5] };
uilist = { ...
	{ 'Style', 'text', 'string', fastif(typeproc, 'Component range', 'Channel range'), 'fontweight', 'bold'  } { 'Style', 'edit', 'string', '1', 'tag', 'chans' } { 'Style', 'pushbutton', 'string', 'Select...', 'callback', 'set(findobj(gcbf, ''tag'', ''chans''), ''string'', int2str(pop_chansel({EEG.chanlocs.labels}, ''withindex'', ''on'')));' }...
	{ 'Style', 'text', 'string', 'Epoch time range [min max] (msec)', 'fontweight', 'bold' } { 'Style', 'edit', 'string', [ num2str(round(EEG.xmin*1000)) ' ' num2str(round(EEG.xmax*1000)) ] } {} ...
	{ 'Style', 'text', 'string', 'Frequency range [min max] (Hz)', 'fontweight', 'bold'} { 'Style', 'edit', 'string', [ '0 ' num2str(nyquist) ] } {} ...
	{ 'Style', 'text', 'string', 'Data window (samples)', 'fontweight', 'bold'} { 'Style', 'edit', 'string', '256' } {} ...
	{ 'Style', 'text', 'string', 'FFT length (samples, power of 2)', 'fontweight', 'bold'} { 'Style', 'edit', 'string', '512' } {} ...
	{ 'Style', 'text', 'string', 'Overlap (samples, < window length)', 'fontweight', 'bold'} { 'Style', 'edit', 'string', '240' } {} ...
    {} {} {} ...
    { 'Style', 'checkbox', 'string', 'Show individual spectrograms'} {} {} ...
    { 'Style', 'checkbox', 'string', 'Multiple ERSGs in same window'} {} {} ...
    { 'Style', 'checkbox', 'string', 'Substitute Z-scores for raw data'} {} {} ...
};

result = inputgui( geometry, uilist, [], fastif(typeproc, 'Plot component ERSG', 'Plot channel ERSG'));

if length( result ) == 0 return; end;

channels = str2num(result{1});
time = str2num(result{2});
freqs = str2num(result{3});
winlen = str2num(result{4});
FFTlen = str2num(result{5});
overlap = str2num(result{6});
ind_spects = result{7};
if (length(channels) > 1) && result{8} multi_ERSGs = 1; else multi_ERSGs = 0; end
z_score = result{9};

if typeproc d = double(EEG.icaact(channels, :, :)); else d = double(EEG.data(channels, :, :)); end

s = spectrogram(d(1, :, 1), winlen, overlap, FFTlen, EEG.srate);
freqlen = size(s, 1);
timelen = size(s, 2);

SG_plot_y_dim = floor(sqrt(EEG.trials));
SG_plot_x_dim = SG_plot_y_dim;
while EEG.trials > (SG_plot_x_dim * SG_plot_y_dim)
    SG_plot_x_dim = SG_plot_x_dim + 1;
end

if multi_ERSGs 
    h = figure('Name', ['ERSG: ' EEG.setname], 'NumberTitle', 'off');
    ERSG_plot_y_dim = floor(sqrt(length(channels)));
    ERSG_plot_x_dim = ERSG_plot_y_dim;
    while length(channels) > (ERSG_plot_x_dim * ERSG_plot_y_dim)
        ERSG_plot_x_dim = ERSG_plot_x_dim + 1;
    end
end
    
zp = abs(EEG.xmin) / (EEG.xmax - EEG.xmin);
zero_sg_idx = floor(timelen * zp);

for i = 1:length(channels)
    ss = zeros(freqlen, timelen);
    for j = 1:EEG.trials
        c = d(i, :, j);
        s = spectrogram(c, winlen, overlap, FFTlen, EEG.srate);
        s = log10(abs(s));
        ss = ss + s;
    end
    ss = ss / EEG.trials;
    ms = mean(ss(:, 1:zero_sg_idx), 2);
    if z_score
        sd = std(ss(:, 1:zero_sg_idx), 0, 2);
    end
    for j = 1:timelen ss(:,j) = ss(:,j) - ms; end
    if z_score
        for j = 1:timelen ss(:,j) = ss(:,j) ./ sd; end
    end

    % narrow to time window
    time2idx = timelen / (EEG.times(end) - EEG.times(1));
    beg_t_idx = round((time(1) - EEG.times(1)) * time2idx);
    if beg_t_idx < 1 beg_t_idx = 1; end
    end_t_idx = round((time(2) - EEG.times(1)) * time2idx);
    if end_t_idx > timelen end_t_idx = timelen; end
    ss = ss(:, beg_t_idx:end_t_idx);

    % narrow to frequency window
    low_f_idx = round((freqs(1) / nyquist) * freqlen);
    if low_f_idx < 1 low_f_idx = 1; end
    high_f_idx = round((freqs(2) / nyquist) * freqlen);
    if high_f_idx > freqlen high_f_idx = freqlen; end
    ss = ss(low_f_idx:high_f_idx, :);

    if multi_ERSGs set(0, 'CurrentFigure', h); subplot(ERSG_plot_y_dim, ERSG_plot_x_dim, i);
    else figure('Name', ['ERSG: ' EEG.setname], 'NumberTitle', 'off'); end
    imagesc(ss);
    
    yticks = get(gca, 'Ytick');
    ylim = get(gca, 'YLim');
    tick2freq = (freqs(2) - freqs(1)) / (ylim(2) - ylim(1));
    label_nums = (yticks' * tick2freq) + freqs(1);
    ylabels = num2str(round(label_nums));

    xticks = get(gca, 'Xtick');
    xlim = get(gca, 'XLim');
    tick2time = (time(2) - time(1)) / (xlim(2) - xlim(1));
    label_nums = (xticks' * tick2time) + time(1);
    xlabels = num2str(round(label_nums));

    time_zero_tick = round(-time(1) / tick2time);
    line([time_zero_tick, time_zero_tick], [ylim(1) ylim(2)], 'Color', 'k', 'LineWidth', 1);

    set(gca, 'YTickLabel', ylabels, 'XTickLabel', xlabels, 'YDir', 'normal');
    if typeproc title(['Component ' num2str(channels(i))], 'Parent', gca);
    else title(['Channel ' num2str(channels(i))], 'Parent', gca); end
    if (i == 1  || ~multi_ERSGs) xlabel('Time (ms)', 'Parent', gca); ylabel('Frequency (Hz)', 'Parent', gca); end
    
    if ind_spects
        if typeproc title_str = ['Trial Spectrograms: Component ' num2str(channels(i))];
        else title_str = ['Trial Spectrograms: Channel ' num2str(channels(i))]; end
        figure('Name', title_str, 'NumberTitle', 'off');
        for j = 1:EEG.trials
            c = d(i, :, j);
            s = spectrogram(c, winlen, overlap, FFTlen, EEG.srate);
            s = log10(abs(s));
            for k = 1:timelen s(:,k) = s(:,k) - ms; end
            s = s(:, beg_t_idx:end_t_idx);
            s = s(low_f_idx:high_f_idx, :);
            subplot(SG_plot_y_dim, SG_plot_x_dim, j);
            imagesc(s);
            line([time_zero_tick, time_zero_tick], [ylim(1) ylim(2)], 'Color', 'k', 'LineWidth', 1);
            set(gca, 'YTickLabel', ylabels, 'XTickLabel', xlabels, 'YDir', 'normal');
            title(['Trial ' num2str(j)], 'Parent', gca)
            if j == 1 xlabel('Time (ms)', 'Parent', gca); ylabel('Frequency (Hz)', 'Parent', gca); end
        end
    end
end

return;

